@@ -90,6 +90,7 @@ def {link.name}(self, _: T.Optional[{link_resolver_name}] = None, /) -> {node_re
 
 
 def build_get_functions_str(node_name: str, exclusive_field_names: T.Set[str]) -> str:
+    exclusive_field_names = sorted(list(exclusive_field_names))
     params_fields_str = ", ".join(
         [f"{f}: T.Optional[T.Any] = None" for f in exclusive_field_names]
     )
@@ -176,7 +177,7 @@ def edgedb_conversion_type_from_prop(prop: Property) -> str:
     return s
 
 
-def build_node_and_resolver(object_type: ObjectType) -> str:
+def build_node_and_resolver(object_type: ObjectType, hydrate: bool) -> str:
     # need to sort props and links by required, exclusive, no default, rest
     object_type.properties.sort(
         key=lambda x: f"{not x.is_computed}-{x.required}-{x.is_exclusive}-{x.default}",
@@ -325,7 +326,14 @@ def {prop.name}(self) -> {prop.type_str}:
         orm_config_str,
     ]
     node_inner_str = "\n".join(node_inner_strs)
-    node_s = f"class {object_type.node_name}(Node[{insert_model_name}]):\n{indent_lines(node_inner_str)}"
+    inherits = (
+        f"Node[{insert_model_name}]"
+        if not hydrate
+        else f"{object_type.node_name}Hydrated"
+    )
+    node_s = (
+        f"class {object_type.node_name}({inherits}):\n{indent_lines(node_inner_str)}"
+    )
 
     # resolver
     resolver_properties_str = f"_node = {object_type.node_name}"
@@ -347,11 +355,17 @@ def {prop.name}(self) -> {prop.type_str}:
     return f"{insert_s}\n{node_s}\n{resolver_s}\n{final_s}"
 
 
-async def build_nodes_and_resolvers(client: edgedb.AsyncIOClient) -> str:
+async def build_nodes_and_resolvers(
+    client: edgedb.AsyncIOClient, nodes_to_hydrate: T.Set[str]
+) -> str:
     object_types = await introspect_objects(client)
     node_strs: T.List[str] = []
     for object_type in object_types:
-        node_strs.append(build_node_and_resolver(object_type))
+        node_strs.append(
+            build_node_and_resolver(
+                object_type, hydrate=object_type.node_name in nodes_to_hydrate
+            )
+        )
     update_forward_refs_str = "\n".join(
         [f"{o.node_name}Insert.update_forward_refs()" for o in object_types]
     )
@@ -367,12 +381,18 @@ class TLSSecurity(str, Enum):
     insecure = "insecure"
 
 
+class NodeConfig(BaseModel):
+    module_path: str
+
+
 class DBConfig(BaseModel):
     vendor: DBVendor
     host: str
     password: str
     port: int = 5656
     tls_security: TLSSecurity
+    hydrate: bool = False
+    nodes: T.Dict[str, NodeConfig] = dict()
 
 
 def build_client(db_config: DBConfig) -> str:
@@ -396,22 +416,58 @@ def validate_output_path(path: Path) -> None:
             os.makedirs(path)
 
 
+def build_hydrate_imports(db_config: DBConfig) -> str:
+    import_strs: T.List[str] = []
+    for node_name, config in db_config.nodes.items():
+        if config.module_path:
+            hydrated_name = f"{node_name}Hydrated"
+            import_strs.append(
+                f"from {config.module_path} import {node_name} as {hydrated_name}"
+            )
+            # idk why i have the part below, was just copying from dgraph_orm, seems unecessary and wrong
+            # import_strs.append(
+            #     f"{hydrated_name}.{CONFIG_NAME}.resolver._node = {hydrated_name}"
+            # )
+    return "\n".join(import_strs)
+
+
+def get_nodes_to_hydrate(db_config: DBConfig) -> T.Set[str]:
+    node_names: T.Set[str] = set()
+    for node_name, config in db_config.nodes.items():
+        if config.module_path:
+            node_names.add(node_name)
+    return node_names
+
+
+async def build_from_config(db_config_d: dict, hydrate: bool = False) -> str:
+    db_config = DBConfig.parse_obj(db_config_d)
+    client = edgedb.create_async_client(
+        tls_security=db_config.tls_security,
+        host=db_config.host,
+        password=db_config.password,
+        port=db_config.port,
+    )
+    imports_str = imports()
+    client_str = build_client(db_config)
+    hydrate_imports = "" if not hydrate else build_hydrate_imports(db_config)
+    enums_str = await build_enums(client)
+    nodes_and_resolvers_str = await build_nodes_and_resolvers(
+        client,
+        nodes_to_hydrate=set() if not hydrate else get_nodes_to_hydrate(db_config),
+    )
+
+    s = "\n".join(
+        [imports_str, client_str, hydrate_imports, enums_str, nodes_and_resolvers_str]
+    )
+    s = format_str(s, mode=FileMode())
+    return s
+
+
 async def generate(config_path: Path, output_path: Path) -> None:
     validate_output_path(output_path)
     for db_name, db_config_d in json.loads(open(config_path).read()).items():
-        db_config = DBConfig.parse_obj(db_config_d)
-        client = edgedb.create_async_client(
-            tls_security=db_config.tls_security,
-            host=db_config.host,
-            password=db_config.password,
-            port=db_config.port,
-        )
-        imports_str = imports()
-        client_str = build_client(db_config)
-        enums_str = await build_enums(client)
-        nodes_and_resolvers_str = await build_nodes_and_resolvers(client)
-
-        s = "\n".join([imports_str, client_str, enums_str, nodes_and_resolvers_str])
-        s = format_str(s, mode=FileMode())
-
+        s = await build_from_config(db_config_d=db_config_d)
         open(output_path / f"{db_name}.py", "w").write(s)
+        if db_config_d.get("hydrate", False):
+            s = await build_from_config(db_config_d=db_config_d, hydrate=True)
+            open(output_path / f"{db_name}_hydrated.py", "w").write(s)
